import { Controller, Get, Next, Param, ParseUUIDPipe, Put, Query, Req, Res } from "@nestjs/common";
import { EsResponseDto } from "src/core/domain/dtos";
import { SearchService } from "../../core/services/common/search.service";
import { response, Response } from "express";

@Controller('papers')
export class PapersController {
    constructor(private searchService: SearchService) {}

    /**
     * 
     * @param query 
     * @param response 
     * @returns a response with a set of matching papers
     */
    @Get('search')
    getByContext(@Query('query') query: string, @Res() response: Response/*, @Query('page') page, @Query('limit') limit*/) {
        return this.searchService.findByContext(query).then(
            (res) => {
                response.status(200).send(res);
            },
            (err) => {
                response.status(err).send();
            }
        );
    }

    /**
     * 
     * @param uuid 
     * @param response 
     * @returns a response with a requested object
     */
    @Get(':uuid')   
    getByID(@Param('uuid', ParseUUIDPipe) uuid: string, @Res() response: Response) {
        return this.searchService.findByID(uuid).then(
            (res) => {
                response.status(200).send(res);
            },
            (err) => {
                response.status(err).send();
            }
        );
    }
}