import { HttpService } from "@nestjs/axios";
import { HttpException, HttpStatus, Injectable, NotAcceptableException, NotFoundException } from "@nestjs/common";
import { map, NotFoundError, take } from "rxjs";
import { EsResponseDto } from "src/core/domain/dtos";

/**
 * Search service provider
 */
@Injectable()
export class SearchService {
    constructor(private readonly httpService: HttpService) {}

    /**
     * Finds a paper by its own ID
     * @param uuid 
     * @returns Elasticsearch hits or an error object
     */
    async findByID(uuid: string): Promise<any> { // Should I change 'object' to specific DTO?
        let es_query = {
            query: {
                query_string: {
                    query: 'id:' + uuid
                }
            }
        }

        // Specify ES_PORT env!
        return new Promise((resolve, reject) => {
            try {
                (this.httpService.get<EsResponseDto>('http://localhost:9200/_search', {
                    data: es_query,
                    headers: {'Content-Type': 'application/json'},
                }))
                .pipe(take(1), map(axiosRes => axiosRes.data))
                .subscribe((res: any) => {
                    if (res.timed_out) {
                        reject(504);
                    }

                    if (!res.hits.hits.length) {
                        reject(404);
                    }

                    resolve(res.hits);
                });
            } catch (error) {
                reject(error);
            }
        });
    }

    /**
     * Finds relevant documents by context using the given query string
     * @param query_str 
     * @returns Elasticsearch hits or an error object
     */
    findByContext(query_str: string): Promise<EsResponseDto> {
        let es_query = {
            query: {
                query_string: {
                    query: query_str,
                    default_field: "content"
                }
            }
        }

        return new Promise((resolve, reject) => {
            try {
                (this.httpService.get<EsResponseDto>('http://localhost:9200/_search', {
                    data: es_query,
                    headers: {'Content-Type': 'application/json'},
                }))
                .pipe(take(1), map(axiosRes => axiosRes.data))
                .subscribe((res: any) => {
                    if (res.timed_out) {
                        reject(504);
                    }

                    if (!res.hits.hits.length) {
                        reject(404);
                    }                

                    resolve(res.hits);
                });
            } catch (error) {
                reject(error);
            }
        });
    }
}